@@ -7,6 +7,7 @@ const Tp = imports.gi.TelepathyGLib;
 
 const AccountsMonitor = imports.accountsMonitor;
 const Lang = imports.lang;
+const Signals = imports.signals;
 
 const ConnectionsDialog = new Lang.Class({
     Name: 'ConnectionsDialog',
@@ -127,9 +128,7 @@ const ConnectionsDialog = new Lang.Class({
     },
 
     _addConnection: function() {
-        this._showConnectionDetailsDialog(null,
-                                          Lang.bind(this,
-                                                    this._createAccount));
+        this._showConnectionDetailsDialog(null);
     },
 
     _removeConnection: function() {
@@ -142,13 +141,132 @@ const ConnectionsDialog = new Lang.Class({
 
     _editConnection: function() {
         let account = this._listBox.get_selected_row()._account;
-        this._showConnectionDetailsDialog(account,
-                                          Lang.bind(this,
-                                                    this._updateAccount,
-                                                    account));
+        this._showConnectionDetailsDialog(account);
     },
 
-    _createAccount: function(params) {
+    _showConnectionDetailsDialog: function(account) {
+        let dialog = new ConnectionDetailsDialog(account);
+        dialog.widget.transient_for = this.widget;
+        dialog.widget.show();
+        dialog.widget.connect('response',
+            function(w, response) {
+                dialog.widget.destroy();
+            });
+    },
+
+    _onDestroy: function() {
+        this._accountsMonitor.disconnect(this._accountAddedId);
+        this._accountsMonitor.disconnect(this._accountRemovedId);
+    }
+});
+
+const ConnectionDetails = new Lang.Class({
+    Name: 'ConnectionDetails',
+
+    _init: function(account) {
+        this._createWidget();
+
+        this._account = account;
+
+        if (account)
+            this._populateFromAccount(account);
+    },
+
+    _getParams: function() {
+        let serverRegEx = /(.*?)(?::(\d{1,5}))?$/;
+        let [, server, port] = this._serverEntry.text.match(serverRegEx);
+
+        let params = {
+            name: this._descEntry.text.length ? this._descEntry.text : server,
+            server: server,
+            account: this._nickEntry.text
+        };
+
+        if (port)
+            params.port = port;
+        if (this._realnameEntry.text)
+            params.fullname = this._realnameEntry.text;
+
+        return params;
+    },
+
+    setCancelVisible: function(visible) {
+        this._cancelButton.visible = visible;
+    },
+
+    reset: function() {
+        this._serverEntry.text = '';
+        this._descEntry.text = '';
+        this._nickEntry.text = '';
+        this._realnameEntry.text = '';
+    },
+
+    _createWidget: function() {
+        let builder = new Gtk.Builder();
+        builder.add_from_resource('/org/gnome/polari/connection-details-dialog.ui');
+
+        this.widget = builder.get_object('connection_details_content');
+        this.widget.unparent();
+
+        this._serverEntry = builder.get_object('server_entry');
+        this._descEntry = builder.get_object('description_entry');
+        this._nickEntry = builder.get_object('nickname_entry');
+        this._realnameEntry = builder.get_object('realname_entry');
+        this.confirmButton = builder.get_object('confirm_button');
+        this._cancelButton = builder.get_object('cancel_button');
+
+        this.confirmButton.connect('clicked',
+                                    Lang.bind(this, this._onConfirmClicked));
+        this._cancelButton.connect('clicked', Lang.bind(this,
+            function() {
+                this.emit('response', Gtk.ResponseType.CANCEL);
+            }));
+
+        this._serverEntry.connect('changed',
+                                  Lang.bind(this, this._updateSensitivity));
+        this._nickEntry.connect('changed',
+                                Lang.bind(this, this._updateSensitivity));
+        this._updateSensitivity();
+    },
+
+    _updateSensitivity: function() {
+        let sensitive = this._serverEntry.get_text_length() > 0 &&
+                        this._nickEntry.get_text_length() > 0;
+        this.confirmButton.sensitive = sensitive;
+    },
+
+    _populateFromAccount: function(account) {
+        let params = account.dup_parameters_vardict().deep_unpack();
+        for (let p in params)
+            params[p] = params[p].deep_unpack();
+
+        let server = params.server || '';
+        let port = params.port || 6667;
+        let nick = params.account || '';
+        let realname = params.fullname || '';
+
+        if (port != 6667)
+            server += ':%d'.format(port);
+
+        this._serverEntry.text = server;
+        this._nickEntry.text = nick;
+        this._realnameEntry.text = realname;
+
+        if (server != account.display_name)
+            this._descEntry.text = account.display_name;
+    },
+
+    _onConfirmClicked: function() {
+        if (this._account)
+            this._updateAccount();
+        else
+            this._createAccount();
+
+        this.emit('response', Gtk.ResponseType.OK);
+    },
+
+    _createAccount: function() {
+        let params = this._getParams();
         let accountManager = Tp.AccountManager.dup();
         let req = new Tp.AccountRequest({ account_manager: accountManager,
                                           connection_manager: 'idle',
@@ -167,7 +285,9 @@ const ConnectionsDialog = new Lang.Class({
             }));
     },
 
-    _updateAccount: function(params, account) {
+    _updateAccount: function() {
+        let params = this._getParams();
+        let account = this._account;
         let oldDetails = account.dup_parameters_vardict().deep_unpack();
         let [details, removed] = this._detailsFromParams(params, oldDetails);
         let vardict = GLib.Variant.new('a{sv}', details);
@@ -198,101 +318,30 @@ const ConnectionsDialog = new Lang.Class({
                 });
 
         return [details, removed];
-    },
-
-    _showConnectionDetailsDialog: function(account, callback) {
-        let dialog = new ConnectionDetailsDialog(account);
-        dialog.widget.transient_for = this.widget;
-        dialog.widget.show();
-        dialog.widget.connect('response',
-            function(w, response) {
-                if (response == Gtk.ResponseType.OK)
-                    callback(dialog.getParams());
-                dialog.widget.destroy();
-            });
-    },
-
-    _onDestroy: function() {
-        this._accountsMonitor.disconnect(this._accountAddedId);
-        this._accountsMonitor.disconnect(this._accountRemovedId);
     }
 });
+Signals.addSignalMethods(ConnectionDetails.prototype);
+
 
 const ConnectionDetailsDialog = new Lang.Class({
     Name: 'ConnectionDetailsDialog',
 
     _init: function(account) {
-        this._createWindow();
+        let title = account ? _("Edit Connection")
+                            : _("New Connection");
+        this.widget = new Gtk.Dialog({ title: title,
+                                       modal: true,
+                                       destroy_with_parent: true });
 
-        if (account) {
-            this.widget.title = _("Edit Connection");
-            this._confirmButton.label = _("A_pply");
+        this._details = new ConnectionDetails(account);
+        this.widget.get_content_area().add(this._details.widget);
 
-            this._populateFromAccount(account);
-        }
-    },
+        this._details.confirmButton.label = account ? _("A_pply")
+                                                    : _("Cr_eate");
 
-    getParams: function() {
-        let serverRegEx = /(.*?)(?::(\d{1,5}))?$/;
-        let [, server, port] = this._serverEntry.text.match(serverRegEx);
-
-        let params = {
-            name: this._descEntry.text.length ? this._descEntry.text : server,
-            server: server,
-            account: this._nickEntry.text
-        };
-
-        if (port)
-            params.port = port;
-        if (this._realnameEntry.text)
-            params.fullname = this._realnameEntry.text;
-
-        return params;
-    },
-
-    _createWindow: function() {
-        let builder = new Gtk.Builder();
-        builder.add_from_resource('/org/gnome/polari/connection-details-dialog.ui');
-
-        this.widget = builder.get_object('connection_details_dialog');
-
-        this._serverEntry = builder.get_object('server_entry');
-        this._descEntry = builder.get_object('description_entry');
-        this._nickEntry = builder.get_object('nickname_entry');
-        this._realnameEntry = builder.get_object('realname_entry');
-        this._confirmButton = builder.get_object('confirm_button');
-
-        this._serverEntry.connect('changed',
-                                  Lang.bind(this, this._updateSensitivity));
-        this._nickEntry.connect('changed',
-                                Lang.bind(this, this._updateSensitivity));
-        this._updateSensitivity();
-    },
-
-    _updateSensitivity: function() {
-        let sensitive = this._serverEntry.get_text_length() > 0 &&
-                        this._nickEntry.get_text_length() > 0;
-        this._confirmButton.sensitive = sensitive;
-    },
-
-    _populateFromAccount: function(account) {
-        let params = account.dup_parameters_vardict().deep_unpack();
-        for (let p in params)
-            params[p] = params[p].deep_unpack();
-
-        let server = params.server || '';
-        let port = params.port || 6667;
-        let nick = params.account || '';
-        let realname = params.fullname || '';
-
-        if (port != 6667)
-            server += ':%d'.format(port);
-
-        this._serverEntry.text = server;
-        this._nickEntry.text = nick;
-        this._realnameEntry.text = realname;
-
-        if (server != account.display_name)
-            this._descEntry.text = account.display_name;
+        this._details.connect('response', Lang.bind(this,
+            function(details, response) {
+                this.widget.response(response);
+            }));
     }
 });
