@@ -73,14 +73,7 @@ const IrcParser = new Lang.Class({
             return;
 
         if (text[0] != '/') {
-            if (text.split('\n').length > MAX_LINES) {
-                let app = Gio.Application.get_default();
-                app.pasteManager.pasteText(text);
-            } else {
-                let type = Tp.ChannelTextMessageType.NORMAL;
-                let message = Tp.ClientMessage.new_text(type, text);
-                this._sendMessage(message);
-            }
+            this._sendOrPasteText(text);
             return;
         }
 
@@ -137,8 +130,7 @@ const IrcParser = new Lang.Class({
                 if (argv.length)
                     log('Excess arguments to JOIN command: ' + argv);
 
-                let account = this._room.channel.connection.get_account();
-
+                let account = this._room.account;
                 let app = Gio.Application.get_default();
                 let action = app.lookup_action('join-room');
                 action.activate(GLib.Variant.new('(ssu)',
@@ -194,8 +186,7 @@ const IrcParser = new Lang.Class({
                 if (argv.length)
                     log('Excess arguments to NICK command: ' + argv);
 
-                let account = this._room.channel.connection.get_account();
-                account.set_nickname_async(nick, Lang.bind(this,
+                this._room.account.set_nickname_async(nick, Lang.bind(this,
                     function(a, res) {
                         try {
                             a.set_nickname_finish(res);
@@ -228,7 +219,7 @@ const IrcParser = new Lang.Class({
                     break;
                 }
 
-                let account = this._room.channel.connection.get_account();
+                let account = this._room.account;
 
                 let app = Gio.Application.get_default();
                 let action = app.lookup_action('message-user');
@@ -239,10 +230,9 @@ const IrcParser = new Lang.Class({
                 break;
             }
             case 'QUIT': {
-                let account = this._room.channel.connection.get_account();
                 let presence = Tp.ConnectionPresenceType.OFFLINE;
                 let message = stripCommand(text);
-                account.request_presence_async(presence, '', message,
+                this._room.account.request_presence_async(presence, 'offline', message,
                     Lang.bind(this, function(a, res) {
                         try {
                             a.request_presence_finish(res);
@@ -257,10 +247,7 @@ const IrcParser = new Lang.Class({
                     output = this._createFeedbackUsage(cmd);
                     break;
                 }
-                let raw = stripCommand(text);
-                let type = Tp.ChannelTextMessageType.NORMAL;
-                let message = Tp.ClientMessage.new_text(type, raw);
-                this._sendMessage(message);
+                this._sendOrPasteText(stripCommand(text));
                 break;
             }
             case 'TOPIC': {
@@ -279,6 +266,18 @@ const IrcParser = new Lang.Class({
             this._app.commandOutputQueue.addNotification(output);
     },
 
+    _sendOrPasteText: function(text) {
+        // auto-paste needs some design; disable for now
+        if (false && text.split('\n').length > MAX_LINES) {
+            let app = Gio.Application.get_default();
+            app.pasteManager.pasteText(text);
+        } else {
+            let type = Tp.ChannelTextMessageType.NORMAL;
+            let message = Tp.ClientMessage.new_text(type, text);
+            this._sendMessage(message);
+        }
+    },
+
     _sendMessage: function(message) {
         this._room.channel.send_message_async(message, 0, Lang.bind(this,
             function(c, res) {
