@@ -21,17 +21,20 @@
 #include "polari-room.h"
 
 struct _PolariRoomPrivate {
+  TpAccount *account;
   TpChannel *channel;
 
   GIcon *icon;
-  char  *id;
+  char  *channel_name;
   char  *display_name;
   char  *topic;
 
   char *self_nick;
 
+  TpHandleType type;
+
   guint self_contact_notify_id;
-  guint identifier_notify_id;
+  guint invalidated_id;
   guint group_contacts_changed_id;
 
   TpProxySignalConnection *properties_changed_id;
@@ -43,6 +46,9 @@ enum
 
   PROP_ID,
   PROP_ICON,
+  PROP_ACCOUNT,
+  PROP_TYPE,
+  PROP_CHANNEL_NAME,
   PROP_CHANNEL,
   PROP_DISPLAY_NAME,
   PROP_TOPIC,
@@ -77,6 +83,31 @@ G_DEFINE_TYPE_WITH_PRIVATE (PolariRoom, polari_room, G_TYPE_OBJECT)
 #define tp_properties_get_all_cb \
         tp_cli_dbus_properties_callback_for_get_all
 
+static void polari_room_set_channel (PolariRoom *room, TpChannel *channel);
+
+
+/**
+ * polari_room_id_from_channel:
+ * @account: a TpAccount
+ * @name: the room name
+ * @type: the room type
+ *
+ * Returns: (transfer full): a room ID corresponding to @account, @name
+ *                           and @type
+ */
+char *
+polari_create_room_id (TpAccount    *account,
+                       const char   *name,
+                       TpHandleType  type)
+{
+  g_return_val_if_fail (TP_IS_ACCOUNT (account), NULL);
+  g_return_val_if_fail (name != NULL, NULL);
+
+  return g_strdup_printf ("%s/%d/%s",
+                          tp_proxy_get_object_path (TP_PROXY (account)),
+                          type, name);
+}
+
 gboolean
 polari_room_should_highlight_message (PolariRoom *room,
                                       TpMessage *message)
@@ -93,6 +124,8 @@ polari_room_should_highlight_message (PolariRoom *room,
 
   if (!priv->channel)
     return FALSE;
+  if (priv->type != TP_HANDLE_TYPE_ROOM)
+    return FALSE;
 
   conn = tp_channel_get_connection (room->priv->channel);
   self = tp_connection_get_self_contact (conn);
@@ -169,27 +202,19 @@ polari_room_compare (PolariRoom *room,
                      PolariRoom *other)
 {
   TpAccount *account1, *account2;
-  TpHandleType type1, type2;
-  TpConnection *conn;
 
   g_return_val_if_fail (POLARI_IS_ROOM (room) && POLARI_IS_ROOM (other), 0);
-  g_return_val_if_fail (room->priv->channel && other->priv->channel, 0);
+  g_return_val_if_fail (room->priv->account && other->priv->account, 0);
 
-  conn = tp_channel_get_connection (room->priv->channel);
-  account1 = tp_connection_get_account (conn);
-
-  conn = tp_channel_get_connection (other->priv->channel);
-  account2 = tp_connection_get_account (conn);
+  account1 = room->priv->account;
+  account2 = other->priv->account;
 
   if (account1 != account2)
     return strcmp (tp_account_get_display_name (account1),
                    tp_account_get_display_name (account2));
 
-  tp_channel_get_handle (room->priv->channel, &type1);
-  tp_channel_get_handle (other->priv->channel, &type2);
-
-  if (type1 != type2)
-    return type1 == TP_HANDLE_TYPE_ROOM ? -1 : 1;
+  if (room->priv->type != other->priv->type)
+    return room->priv->type == TP_HANDLE_TYPE_ROOM ? -1 : 1;
 
   return strcmp (room->priv->display_name, other->priv->display_name);
 }
@@ -214,6 +239,9 @@ update_self_nick (PolariRoom *room)
 
   g_clear_pointer (&priv->self_nick, g_free);
 
+  if (!room->priv->channel)
+    return;
+
   conn = tp_channel_get_connection (room->priv->channel);
   self = tp_connection_get_self_contact (conn);
 
@@ -231,14 +259,11 @@ static void
 update_identifier (PolariRoom *room)
 {
   PolariRoomPrivate *priv = room->priv;
-  const char *id = NULL;
-
-  if (priv->channel)
-    id = tp_channel_get_identifier (priv->channel);
 
   g_clear_pointer (&priv->display_name, g_free);
-  if (id)
-    priv->display_name = g_strdup (id + (id[0] == '#' ? 1 : 0));
+  if (priv->channel_name)
+    priv->display_name = g_strdup (priv->channel_name +
+                                   (priv->channel_name[0] == '#' ? 1 : 0));
 
   g_object_notify_by_pspec (G_OBJECT (room), props[PROP_DISPLAY_NAME]);
 }
@@ -250,9 +275,7 @@ update_icon (PolariRoom *room)
 
   g_clear_object (&priv->icon);
 
-  if (priv->channel &&
-      !tp_proxy_has_interface_by_id (TP_PROXY (priv->channel),
-                                     TP_IFACE_QUARK_CHANNEL_INTERFACE_GROUP))
+  if (priv->type == TP_HANDLE_TYPE_CONTACT)
     priv->icon = g_themed_icon_new ("avatar-default-symbolic");
 
   g_object_notify_by_pspec (G_OBJECT (room), props[PROP_ICON]);
@@ -266,14 +289,6 @@ on_self_contact_notify (GObject    *object,
   update_self_nick (POLARI_ROOM (user_data));
 }
 
-static void
-on_identifier_notify (GObject    *object,
-                      GParamSpec *pspec,
-                      gpointer    user_data)
-{
-  update_identifier (POLARI_ROOM (user_data));
-}
-
 static void
 on_group_contacts_changed (TpChannel  *channel,
                            GPtrArray  *added,
@@ -331,6 +346,16 @@ on_group_contacts_changed (TpChannel  *channel,
   g_free (message);
 }
 
+static void
+on_channel_invalidated (TpProxy  *channel,
+                        guint     domain,
+                        int       code,
+                        char     *message,
+                        gpointer  user_data)
+{
+  polari_room_set_channel (POLARI_ROOM (user_data), NULL);
+}
+
 static void
 update_subject (PolariRoom *room,
                 GHashTable *properties)
@@ -375,6 +400,112 @@ properties_changed (TpProxy *proxy,
   update_subject (POLARI_ROOM (data), changed);
 }
 
+static void
+on_contact_info_ready (GObject      *source,
+                       GAsyncResult *res,
+                       gpointer      data)
+{
+  PolariRoom *room = data;
+  PolariRoomPrivate *priv = room->priv;
+  GList *infos, *l;
+
+  infos = tp_contact_dup_contact_info (TP_CONTACT (source));
+  for (l = infos; l; l = l->next)
+    {
+      TpContactInfoField *f = l->data;
+
+      if (strcmp (f->field_name, "fn") != 0)
+        continue;
+
+      if (f->field_value && *f->field_value)
+        {
+          g_free (priv->topic);
+          priv->topic = g_strdup (*f->field_value);
+
+          g_object_notify_by_pspec (G_OBJECT (room), props[PROP_TOPIC]);
+        }
+      break;
+    }
+  tp_contact_info_list_free (infos);
+}
+
+static void
+polari_room_set_account (PolariRoom *room,
+                         TpAccount  *account)
+{
+  PolariRoomPrivate *priv;
+
+  g_return_if_fail (POLARI_IS_ROOM (room));
+  g_return_if_fail (TP_IS_ACCOUNT (account));
+
+  priv = room->priv;
+
+  if (priv->account == account)
+    return;
+
+  if (priv->account)
+    g_object_unref (priv->account);
+  priv->account = g_object_ref (account);
+
+  g_object_notify_by_pspec (G_OBJECT (room), props[PROP_ACCOUNT]);
+}
+
+static void
+polari_room_set_type (PolariRoom *room,
+                      int         type)
+{
+  PolariRoomPrivate *priv;
+
+  g_return_if_fail (POLARI_IS_ROOM (room));
+
+  priv = room->priv;
+
+  if (priv->type == type)
+    return;
+
+  priv->type = type;
+
+  g_object_freeze_notify (G_OBJECT (room));
+
+  g_object_notify_by_pspec (G_OBJECT (room), props[PROP_TYPE]);
+  update_icon (room);
+
+  g_object_thaw_notify (G_OBJECT (room));
+}
+
+static void
+polari_room_set_channel_name (PolariRoom *room,
+                              const char *channel_name)
+{
+  PolariRoomPrivate *priv;
+
+  g_return_if_fail (POLARI_IS_ROOM (room));
+
+  priv = room->priv;
+
+  if (priv->channel_name)
+    g_free (priv->channel_name);
+  priv->channel_name = g_strdup (channel_name);
+
+  update_identifier (room);
+
+  g_object_notify_by_pspec (G_OBJECT (room), props[PROP_CHANNEL_NAME]);
+}
+
+static gboolean
+check_channel (PolariRoom *room,
+               TpChannel  *channel)
+{
+  PolariRoomPrivate *priv = room->priv;
+  TpAccount *account;
+
+  g_return_val_if_fail (priv->account != NULL && priv->channel_name != NULL, FALSE);
+
+  account = tp_connection_get_account (tp_channel_get_connection (channel));
+  return account == priv->account &&
+         g_strcmp0 (tp_channel_get_identifier (channel), priv->channel_name) == 0;
+}
+
 static void
 polari_room_set_channel (PolariRoom *room,
                          TpChannel  *channel)
@@ -391,8 +522,8 @@ polari_room_set_channel (PolariRoom *room,
 
   if (priv->channel)
     {
-      g_signal_handler_disconnect (priv->channel, priv->identifier_notify_id);
       g_signal_handler_disconnect (priv->channel, priv->group_contacts_changed_id);
+      g_signal_handler_disconnect( priv->channel, priv->invalidated_id);
       g_signal_handler_disconnect (tp_channel_get_connection (priv->channel),
                                    priv->self_contact_notify_id);
 
@@ -401,14 +532,21 @@ polari_room_set_channel (PolariRoom *room,
       g_clear_object (&priv->channel);
     }
 
-  if (channel)
+  if (channel && check_channel (room, channel))
     {
+      TpContact *target = tp_channel_get_target_contact (channel);
+
       priv->channel = g_object_ref (channel);
 
-      if (priv->id == NULL)
-        priv->id = g_strdup (tp_proxy_get_object_path (TP_PROXY (channel)));
-
-      tp_cli_dbus_properties_call_get_all (channel, -1,
+      /* If we have a target contact, the chat is private; assume that this
+       * is mutually exclusive with subject/topic support and look for the
+       * contact's full name as topic.
+       */
+      if (target)
+        tp_contact_request_contact_info_async (target, NULL,
+                                               on_contact_info_ready, room);
+      else
+        tp_cli_dbus_properties_call_get_all (channel, -1,
                                      TP_IFACE_CHANNEL_INTERFACE_SUBJECT,
                                      (tp_properties_get_all_cb)subject_get_all,
                                      room, NULL, NULL);
@@ -418,12 +556,12 @@ polari_room_set_channel (PolariRoom *room,
         g_signal_connect (tp_channel_get_connection (channel),
                           "notify::self-contact",
                           G_CALLBACK (on_self_contact_notify), room);
-      priv->identifier_notify_id =
-        g_signal_connect (channel, "notify::identifier",
-                          G_CALLBACK (on_identifier_notify), room);
       priv->group_contacts_changed_id =
         g_signal_connect (channel, "group-contacts-changed",
                           G_CALLBACK (on_group_contacts_changed), room);
+      priv->invalidated_id =
+        g_signal_connect (channel, "invalidated",
+                          G_CALLBACK (on_channel_invalidated), room);
       priv->properties_changed_id =
         tp_cli_dbus_properties_connect_to_properties_changed (
                                  channel,
@@ -431,15 +569,13 @@ polari_room_set_channel (PolariRoom *room,
                                  room, NULL, NULL, NULL);
     }
 
-    g_object_freeze_notify (G_OBJECT (room));
+  g_object_freeze_notify (G_OBJECT (room));
 
-    update_self_nick (room);
-    update_identifier (room);
-    update_icon (room);
+  update_self_nick (room);
 
-    g_object_notify_by_pspec (G_OBJECT (room), props[PROP_CHANNEL]);
+  g_object_notify_by_pspec (G_OBJECT (room), props[PROP_CHANNEL]);
 
-    g_object_thaw_notify (G_OBJECT (room));
+  g_object_thaw_notify (G_OBJECT (room));
 }
 
 static void
@@ -453,11 +589,23 @@ polari_room_get_property (GObject    *object,
   switch (prop_id)
     {
     case PROP_ID:
-      g_value_set_string (value, priv->id);
+      g_value_take_string (value,
+                           polari_create_room_id (priv->account,
+                                                  priv->channel_name,
+                                                  priv->type));
       break;
     case PROP_ICON:
       g_value_set_object (value, priv->icon);
       break;
+    case PROP_ACCOUNT:
+      g_value_set_object (value, priv->account);
+      break;
+    case PROP_TYPE:
+      g_value_set_int (value, priv->type);
+      break;
+    case PROP_CHANNEL_NAME:
+      g_value_set_string (value, priv->channel_name);
+      break;
     case PROP_CHANNEL:
       g_value_set_object (value, priv->channel);
       break;
@@ -484,6 +632,15 @@ polari_room_set_property (GObject      *object,
 
   switch (prop_id)
     {
+    case PROP_ACCOUNT:
+      polari_room_set_account (room, g_value_get_object (value));
+      break;
+    case PROP_TYPE:
+      polari_room_set_type (room, g_value_get_int (value));
+      break;
+    case PROP_CHANNEL_NAME:
+      polari_room_set_channel_name (room, g_value_get_string (value));
+      break;
     case PROP_CHANNEL:
       polari_room_set_channel (room, g_value_get_object (value));
       break;
@@ -495,7 +652,10 @@ polari_room_set_property (GObject      *object,
 static void
 polari_room_dispose (GObject *object)
 {
+  PolariRoomPrivate *priv = POLARI_ROOM (object)->priv;
+
   polari_room_set_channel (POLARI_ROOM (object), NULL);
+  g_clear_object (&priv->account);
   G_OBJECT_CLASS (polari_room_parent_class)->dispose (object);
 }
 
@@ -504,7 +664,7 @@ polari_room_finalize (GObject *object)
 {
   PolariRoomPrivate *priv = POLARI_ROOM (object)->priv;
 
-  g_clear_pointer (&priv->id, g_free);
+  g_clear_pointer (&priv->channel_name, g_free);
   g_clear_pointer (&priv->display_name, g_free);
 
   G_OBJECT_CLASS (polari_room_parent_class)->finalize (object);
@@ -548,12 +708,35 @@ polari_room_class_init (PolariRoomClass *klass)
                          G_TYPE_ICON,
                          G_PARAM_READABLE);
 
+  props[PROP_ACCOUNT] =
+    g_param_spec_object ("account",
+                         "Account",
+                         "Account",
+                         TP_TYPE_ACCOUNT,
+                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
+
+  props[PROP_TYPE] =
+    g_param_spec_int ("type",
+                       "Type",
+                       "Type",
+                       TP_HANDLE_TYPE_NONE,
+                       TP_HANDLE_TYPE_GROUP,
+                       TP_HANDLE_TYPE_ROOM,
+                       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
+
+  props[PROP_CHANNEL_NAME] =
+    g_param_spec_string ("channel-name",
+                         "Channel name",
+                         "Channel name",
+                         NULL,
+                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
+
   props[PROP_CHANNEL] =
     g_param_spec_object ("channel",
                          "Channel",
                          "Channel",
                          TP_TYPE_CHANNEL,
-                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
+                         G_PARAM_READWRITE);
 
   g_object_class_install_properties (object_class, LAST_PROP, props);
 
