@@ -14,6 +14,9 @@ const UserListPopover = new Lang.Class({
     _init: function() {
         this._createWidget();
 
+        this.widget.connect('closed', Lang.bind(this, function() {
+            this._entry.text = '';
+        }));
         this.widget.connect('map', Lang.bind(this, function() {
             this._revealer.transition_duration = 0;
         }));
@@ -60,9 +63,9 @@ const UserListPopover = new Lang.Class({
                 let [, keyval] = event.get_keyval();
                 if (keyval == Gdk.KEY_Escape) {
                     this._entry.text = '';
-                    return true;
+                    return Gdk.EVENT_STOP;
                 }
-                return false;
+                return Gdk.EVENT_PROPAGATE;
             }));
         this._revealer.add(this._entry);
 
